%
% Soft and Living Materials, ETH Zuerich
% R.W. Style and A.L. Torzynski
% version 2020.07.16
%
% FILE: capillary_length_sessiledrop.m
%
% FUNCTIONS:    capillary_length_sessiledrop
%               findbstar_sessiledrop
%               dropfitter_sessiledrop
%               solvediffeq_sessiledrop
%               diffeq_sessiledrop
%               solverevent_sessiledrop
%
% Functions to calculate the surface tension between two fluids.
%
% The theoretical shape of the drop depends on two parameters. The dimensionless theoretical shape depends on one parameter, 'b'. 
% The code solves the system of differential equations giving the dimensionless theoretical shape while optimizing 'b' 
% to fit the experimental data points clicked by the user. Once the right shape is found, the scale factor between 
% the dimensionless theoretical shape and the theoretical shape yields the surface tension between the two fluids.
%
% References: 
% [1] del Rio and Neumann, Journal of colloid and interface science 196, 136-147 (1997) - doi.org/10.1006/jcis.1997.5214


function Lc = capillary_length_sessiledrop(filename, scale, N)

% Calculate the surface tension between two fluids from the theoretical, dimensionless shape of a sessile drop.
%
% OLD NAME: -
%
% INPUT:    'filename'= path to image file
%           'scale'= scale of the image in pixel/m
%           'N' = number of points to manually click around the outside of the
%           droplet
%
% OUTPUT:   'Lc'= capillary length in m
%
% To calculate the surface tension in physical units
% gamma=Lc^2*deltarho*g; where g =9.81 and deltarho is the density
% difference in kg/m^3


% Plot image
im=imread(filename);
imagesc(im)
hold on
title('Click left-most, top, and right-most points of droplet')
% User input: 
[x,y]=ginput(3);

% Calculate the drop parameters from the user input
top=y(2);
bottom=(y(1)+y(3))/2;
diameter=(x(3)-x(1));
middle=(x(3)+x(1))/2;
radius=diameter/2;
height=abs(top-bottom);
crit_ratio=height/radius;

% User input: N test points to be fitted with the data
title(['Click ',int2str(N),' points around the outline. Cluster near high curvature for more accuracy'])
[X,Y]=ginput(N);

% Change the reference for data point coordinates
testr=X-middle;
testz=abs(top-Y);

% Find the parameter 'beta' i.e. the theoretical shape that best fits the experimental points
beta=findbstar_sessiledrop(crit_ratio,testr,-testz,radius);

% Calculate the theoretical drop shape for c=1 and b=bstar
sol=solvediffeq_sessiledrop(beta,1,15,crit_ratio);

% Extract the point coordinates from solver result
s=linspace(0,max(sol.x));
x=deval(sol,s,2);
z=deval(sol,s,3);

% Extract the point coordinates at maximum width
x_cl=sol.ye(2,1);

% Rescale the coordinates of theoretical points to match experimental data
Lc=radius/x_cl;
x=x*Lc;
z=z*Lc;
Lc=Lc/scale;

% Add the theoretical shape to the plot
plot(x+middle,-z+top,'ro')
plot(-x+middle,-z+top,'ro')

end


function bstar = findbstar_sessiledrop(crit_ratio, testr, testz, radius)

% Find the parameter bstar that best fits the shape of the drop in the image.
%
% OLD NAME: sessile_drop_mk3
%
% INPUT:    'crit_ratio'= aspect ratio of the drop
%           'testr'= horizontal coordinates of the test points
%           'testz'= vertical coordinates of the test points
%           'radius'= radius of the drop
%
% OUTPUT:   'bstar'= parameter describing the theoretical shape that best fits the test points


% Minimise the function 'dropfitter_sessiledrop' with respect to parameter 'b', with -1 < b < 0.001
bstar=fminbnd(@(b) dropfitter_sessiledrop(b,testr,testz,radius,crit_ratio),-1,0.001);

end


function checkval = dropfitter_sessiledrop(b, x_test, z_test, radius, crit_ratio)

% Quantify how well a given theoretical drop shape matches experimental data points.
%
% OLD NAME: drop_fitter2
%
% INPUT:    'b'= parameter describing the dimensionless theoretical shape
%           'x_test'= x-coordinates of experimental data points
%           'z_test'= z-coordinates of experimental data points
%           'radius'= radius of the drop
%           'crit_ratio'= aspect ratio of the drop
%
% OUTPUT:   'checkval'= test value to be minimized by changing b


% Calculate the theoretical drop shape for c=1 and given b
sol=solvediffeq_sessiledrop(b,1,15,crit_ratio);

% Extract the point coordinates from solver result
s=linspace(0,max(sol.x));
x=deval(sol,s,2);
z=deval(sol,s,3);

% Extract the point coordinates at maximum width
x_cl=sol.ye(2,1);

% Rescale the coordinates of theoretical points to match experimental data
lambda=radius/x_cl;
x=x*lambda;
z=z*lambda;

% Calculate what the x-coordinates of experimental data points should be, according to the theoretical shape, given their z-coordinates
xtestpos=interp1(z,x,z_test,'pchip');
% EDIT: replaced 'cubic' with 'pchip' to remove warning messages

% Define the value to be minimized by comparing the theoretical and experimental x-coordinates
checkval=sum(abs(x_test-xtestpos));

end


function sol = solvediffeq_sessiledrop(b, c, length, crit_ratio)

% Integrate the system of differential equations.
%
% OLD NAME: plot_pendant_better
%
% INPUT:    'b'= curvature at the origin
%           'c'= capillary constant
%           'length'= maximum arc length to integrate
%           'crit_ratio'= aspect ratio of the drop
%
% OUTPUT:   'sol'= list of coordinates for points along the integrated drop shape
%               'sol.x'= arc length
%               'sol.y(:,1)'= theta
%               'sol.y(:,2)'= x
%               'sol.y(:,3)'= z
%               'sol.ye'= coordinates of special points


% As the solver calculates the shape, run tests on the points to:
%   - find the point of maximum width
%   - stop the integration when decreasing theta is equal to -pi
Call_solverevent_sessiledrop=@(s,y) solverevent_sessiledrop(s,y,crit_ratio);
options=odeset('Events',Call_solverevent_sessiledrop);

% Integrate the system of differential equations between s=0 and s=length, with initial values theta=0 and x=0 and z=0 at s=0
sol = ode45(@(s,y) diffeq_sessiledrop(s,y,b,c),[0 length],[0 0 0],options);

end


function dydx = diffeq_sessiledrop(s, y, bb, cc)

% Define the system of differential equations to be integrated.
%
% OLD NAME: pendant_drop
%
% INPUT:    's'= arc length
%           'y'= 3x1 vector containing the variables relevant to the integration
%               'y(1)'= theta
%               'y(2)'= x
%               'y(3)'= z
%           'bb'= curvature at the origin
%           'cc'= capillary constant
%
% OUTPUT:   'dydx'= 3x1 vector containing the derivatives of the equation system
%               'dydx(1)'= d(theta)/ds
%               'dydx(2)'= d(x)/ds
%               'dydx(3)'= d(z)/ds


% Equations 1c and 1g [1]
if y(2)==0
    dthetads=2*bb+cc*y(3)-bb;
else
    dthetads=2*bb+cc*y(3)-sin(y(1))/y(2);
end

% Equation 1a [1]
dxds=cos(y(1));

% Equation 1b [1]
dzds=sin(y(1));

% Differential equations system
dydx=[dthetads dxds dzds]';

end


function [lookfor,stop,direction] = solverevent_sessiledrop(s,y,crit)

% Instructions for the solver to:
%   - note the point of maximum width i.e. when increasing abs(z/x) equals the aspect ratio of the drop
%   - stop the integration when decreasing theta is equal to -pi
%
% OLD NAME: pendevent
%
% INPUT:    's'= arc length
%           'y'= 3x1 vector containing the variables relevant to the integration
%               'y(1)'= theta
%               'y(2)'= x
%               'y(3)'= z
%           'crit'= aspect ratio of the drop
%
% OUTPUT:   see: ch.mathworks.com/help/matlab/ref/odeset.html#d117e994192


lookfor=[abs(y(3)/y(2))-crit y(1)+pi];
stop=[0 1];
direction=[1 -1];

end
